
Preising = {};

function Preising.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Preising:load(xmlFile)

	self.shovelInRange = SpecializationUtil.callSpecializationsFunction("shovelInRange");
	
	self.connectCollisions = SpecializationUtil.callSpecializationsFunction("connectCollisions");
	self.anim = SpecializationUtil.callSpecializationsFunction("anim");
	self.Go = {}; 
	self.Done = {};
	self.charId = {};
	self.clipIndex = {};
	self.CheckDone = {};
	self.moveColli = {}; 
    self.animParts = {}
	self.collisionArm = {}; 
	local count = getXMLInt(xmlFile, "vehicle.animParts#count");
	local part = self.animParts;
    for i=1, count do
		part[i] = {};
        local partname = string.format("vehicle.animParts.part".."%d", i);
	    local nameR = getXMLString(xmlFile, partname.."#name");
		self.charId[nameR] = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#rootNode"));
		self.clipIndex[nameR] = getXMLString(xmlFile, partname.."#animationClip");
		self.CheckDone[nameR] = false;
		self.moveColli[nameR] = getXMLString(xmlFile, partname.."#moveColli");
		if self.moveColli[nameR] ~= nil then
			local Collision = {};
			Collision.collision = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collision"));
			Collision.collisionAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collsionAttacher"));
			Collision.armAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.. "#armAttacher"));
			Collision.index = 0;
			self.collisionArm[nameR] = Collision;
			self:connectCollisions(nameR);
		end; 
	end; 
	
	
	self.dustParticleSystems = {};
    local dustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.dustParticleSystems#count"), 0);
    for i=1, dustParticleSystemCount do
        local namei = string.format("vehicle.dustParticleSystems.part%d", i);
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, namei .. "#index"));
        Utils.loadParticleSystem(xmlFile, self.dustParticleSystems, namei, nodei, false, nil, self.baseDirectory)			
    end;
	
	self.chargeParticle = {};
    local i = 0;
    while true do
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.chargeParticle#index"));
        local key = string.format("vehicle.chargeParticle.chargeParticle(%d)", i);
        local t = getXMLString(xmlFile, key.."#type");
		if t == nil then
            break;
        end
        local currentPS = {};
		local particleNode = Utils.loadParticleSystem(xmlFile, currentPS, key, nodei, false, "$data/vehicles/particleSystems/trailerDischargeParticleSystem.i3d", self.baseDirectory);
		self.chargeParticle[t] = currentPS;
		i = i +1;
    end;
	
	self.keyType = InputBinding.getButtonKeyName;
	self.keyName = "Taste"; 
	
	self.isShovel = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.isShovel#value"), false);
end;


function Preising:update(dt)

	if self.attacherVehicle then
		self:shovelInRange();
		
		if InputBinding.hasEvent(InputBinding.Preising_greif) and self:getIsActiveForInput() then
			self.Go.greif = not self.Go.greif; 
			self.Done.greif = true; 
		end; 
		
	end; 
	
	if self.Go.greif ~= nil and self.Done.greif ~= false then
		self:anim("greif", false); 
	end; 

end;


function Preising:shovelInRange()
    local nearestDistance = 4.0;
	self.inRange = false; 
	local px, py, pz = getWorldTranslation(self.rootNode);
	Utils.setEmittingState(self.savePS, false);
	for i=1, table.getn(g_currentMission.attachables) do
		local vx, vy, vz = getWorldTranslation(g_currentMission.attachables[i].rootNode);
		local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
		local attachable = g_currentMission.attachables[i];	
		if distance < nearestDistance and self.filllevel ~= 0 and self.currentFillType ~= nil and attachable:allowFillType(self.currentFillType, true) and attachable.setFillLevel and self.setFillLevel and self.Go.greif and self.CheckDone.greif then
			for fruitType,v in pairs(attachable.fillTypes) do
				if fruitType == self.currentFillType then
					local x,y,z = localDirectionToWorld(self.rootNode, 0,0,1);
					local x,yShovel,z = worldDirectionToLocal(self.attacherVehicle.rootNode, x,y,z);
					if yShovel >= 0.0  then
						if attachable.setFillLevel ~= nil and attachable.rootNode ~= self.rootNode then
							if attachable.fillLevel == attachable.capacity then
								Utils.setEmittingState(self.savePS, false);
							else
								if self.fillLevel ~= 0 and self.setFillLevel ~= nil then
									self.savePS = self.chargeParticle[FruitUtil.fruitIndexToDesc[self.currentFillType].name];
									Utils.setEmittingState(self.savePS, true);
								end; 										
								if self.fillLevel < -0 then
									self.fillLevel = 0; 
									self:setFillLevel(0.0, FruitUtil.FRUITTYPE_UNKNOWN); 
								end;
									
								if self.fillLevel > 0 then
									self.fillLevel = self.fillLevel - yShovel * 100;
									local newLevel = attachable.fillLevel + yShovel * 100;
									attachable:setFillLevel(newLevel, self.currentFillType);	
								end; 
								
								self.inRange = true; 
							end; 
						end; 
					end; 
				end;
			end;
		end; 
    end;
	
	if not self.inRange or self.fillLevel == 0 or not self.Go.greif or not self.CheckDone.greif then
		Utils.setEmittingState(self.savePS, false);
	end; 
end;


function Preising:delete()
	Utils.setEmittingState(self.savePS, false);
	Utils.setEmittingState(self.chargeParticle, false)
	Utils.deleteParticleSystem(self.chargeParticle)
end;


function Preising:anim(varName, loopCheck)
	if self.moveColli[varName] ~= nil then
		local Collision = self.collisionArm[varName];
		setJointFrame(Collision.index, 0, Collision.armAttacher);
	end;
	if self.Go[varName] == true and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, 1);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
			disableAnimTrack(charId, 0);	
			self.Done[varName] = false;	
			self.CheckDone[varName] = true;	
		end; 
	elseif self.Go[varName] == false and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		if loopCheck == true then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
		end; 	
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, -1);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) <= 0 then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
			self.CheckDone[varName] = false;
		end;
	end;
end; 


function Preising:connectCollisions(varName)
	local Collision = self.collisionArm[varName];
	local constr = JointConstructor:new();
	constr:setActors(self.rootNode, Collision.collision);
	constr:setJointTransforms(Collision.armAttacher, Collision.collisionAttacher);
	for i=1, 3 do		            
		constr:setRotationLimit(i-1, 0, 0, 0);
		constr:setTranslationLimit(i-1, true, 0, 0);
	end;
	Collision.index = constr:finalize();
end;

function Preising:anim(varName, loopCheck)
	if self.moveColli[varName] ~= nil then
		local Collision = self.collisionArm[varName];
		setJointFrame(Collision.index, 0, Collision.armAttacher);
	end;
	if self.Go[varName] == true and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, 1);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
			disableAnimTrack(charId, 0);	
			self.Done[varName] = false;	
			self.CheckDone[varName] = true;	
		end; 
	elseif self.Go[varName] == false and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		if loopCheck == true then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
		end; 	
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, -1);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) <= 0 then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
			self.CheckDone[varName] = false;
		end;
	end;
end; 

function Preising:connectCollisions(varName)
	local Collision = self.collisionArm[varName];
	local constr = JointConstructor:new();
	constr:setActors(self.rootNode, Collision.collision);
	constr:setJointTransforms(Collision.armAttacher, Collision.collisionAttacher);
	for i=1, 3 do		            
		constr:setRotationLimit(i-1, 0, 0, 0);
		constr:setTranslationLimit(i-1, true, 0, 0);
	end;
	Collision.index = constr:finalize();
end;

function Preising:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Preising:keyEvent(unicode, sym, modifier, isDown)
	if isDown and sym==Input.KEY_8 then
		self.keySet = not self.keySet; 
		if self.keySet then
			self.keyType = InputBinding.getButtonName;
			self.keyName = "Button"; 
		else
			self.keyType = InputBinding.getButtonKeyName;
			self.keyName = "Taste"; 
		end; 
	end; 
end;

function Preising:draw()
	if self.Go.greif then
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.Preising_greif).. ": " ..g_i18n:getText("Preising_1")); 
	else 
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.Preising_greif).. ": " ..g_i18n:getText("Preising_2")); 
	end; 
end;